/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999,2000 Hiroyuki Yamamoto
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtkmain.h>

#include "automaton.h"

Automaton *automaton_create(gint num)
{
	Automaton *atm;

	g_return_val_if_fail(num > 0, NULL);

	atm = g_new0(Automaton, 1);
	atm->max = num - 1;
	atm->state = g_new0(AtmState, num);

	return atm;
}

void automaton_destroy(Automaton *atm)
{
	if (!atm) return;

	g_free(atm->state);
	g_free(atm);
}

#if 0
static const char *
mk_cond_str ( GIOCondition cond )
{
    if ( (cond & G_IO_OUT) && (cond & G_IO_IN) )
        return "in+out";
    if ( (cond & G_IO_OUT) )
        return "out";
    if ( (cond & G_IO_IN) )
        return "in";
    return "-";
}
#endif

void automaton_input_cb(gpointer data, SockInfo *sockinfo,
			GIOCondition condition)
{
	Automaton *atm = (Automaton *)data;
	gint next;

 again:
	if (atm->timeout_tag > 0) {
		gtk_timeout_remove(atm->timeout_tag);
		atm->timeout_tag = 0;
		atm->elapsed = 0;
	}
        /* Hmmm: Why should we remove and install this input handlers
         * they should be protected against recursive calls by glib */
	/*gdk_input_remove(atm->tag);*/
	/*atm->tag = 0;*/

	next = atm->state[atm->num].handler(sockinfo, atm->data);
	if (next >= 0 && next <= atm->max && next != atm->num) {
		atm->num = next;
		/*atm->tag = gdk_input_add(source,
					 atm->state[atm->num].condition,
					 automaton_input_cb,
					 data);*/
#ifdef HAVE_DOSISH_SYSTEM
                /* GLib does currently not provide a way to wait for
                 * G_IO_OUT, so we would block.  The workaround we use is 
                 * to check for G_IO_OUT condition here and restart */
                if ( (atm->state[atm->num].condition & G_IO_OUT) ) 
                    goto again;
#endif                
                /* because our sock_read() returns only entire lines,
                 * it might buffer more characters.  The g_io_stuff does
                 * not know about it and therefore we have to check here
                 * whether we should process the rest. 
                 * FIXME: Maybe we can install an idle handler for it
                 */
                if ( (atm->state[atm->num].condition & G_IO_IN)
                     && sock_bytes_pending (sockinfo) ) 
                    goto again;


	} else {
            atm->terminate(sockinfo, data);
	}
}




