/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999,2000 Hiroyuki Yamamoto
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "defs.h"

#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "intl.h"
#include "main.h"
#include "prefs.h"
#include "prefs_common.h"
#include "mainwindow.h"
#include "summaryview.h"
#include "messageview.h"
#include "manage_window.h"
#include "filter.h"
#include "foldersel.h"
#include "menu.h"
#include "codeconv.h"
#include "utils.h"
#include "gtkutils.h"
#include "alertpanel.h"
#include "folder.h"

PrefsCommon prefs_common;

static PrefsDialog dialog;

static struct Receive {
	GtkWidget *checkbtn_incext;
	GtkWidget *entry_incext;
	GtkWidget *button_incext;

	GtkWidget *checkbtn_local;
	GtkWidget *checkbtn_filter_on_inc;
	GtkWidget *entry_spool;

	GtkWidget *checkbtn_autochk;
	GtkWidget *spinbtn_autochk;
	GtkObject *spinbtn_autochk_adj;

	GtkWidget *checkbtn_chkonstartup;

	GtkWidget *spinbtn_maxarticle;
	GtkObject *spinbtn_maxarticle_adj;
} receive;

static struct Send {
	GtkWidget *checkbtn_sendext;
	GtkWidget *entry_sendext;
	GtkWidget *button_sendext;

	GtkWidget *checkbtn_savemsg;
	GtkWidget *checkbtn_queuemsg;

	GtkWidget *optmenu_charset;
} send;

static struct Compose {
	GtkWidget *checkbtn_quote;
	GtkWidget *entry_quotemark;
	GtkWidget *text_quotefmt;
	GtkWidget *checkbtn_autosig;
	GtkWidget *entry_sigsep;

	GtkWidget *spinbtn_linewrap;
	GtkObject *spinbtn_linewrap_adj;
	GtkWidget *checkbtn_wrapquote;
	GtkWidget *checkbtn_wrapatsend;
} compose;

static struct Filter {
	GtkWidget *hdr_combo1;
	GtkWidget *hdr_combo2;
	GtkWidget *hdr_entry1;
	GtkWidget *hdr_entry2;
	GtkWidget *key_entry1;
	GtkWidget *key_entry2;
	GtkWidget *pred_combo1;
	GtkWidget *pred_combo2;
	GtkWidget *pred_entry1;
	GtkWidget *pred_entry2;
	GtkWidget *op_combo;
	GtkWidget *op_entry;

	GtkWidget *dest_entry;
	GtkWidget *regex_chkbtn;

	GtkWidget *destsel_btn;
	GtkWidget *dest_radiobtn;
	GtkWidget *notrecv_radiobtn;

	GtkWidget *cond_clist;
} filter;

static struct Display {
	GtkWidget *entry_msgfont;
	GtkWidget *button_msgfont;

	GtkWidget *chkbtn_folder_unread;

	GtkWidget *chkbtn_enablecol;
	GtkWidget *button_edit_col;

	GtkWidget *chkbtn_transhdr;
	GtkWidget *chkbtn_mbalnum;
	GtkWidget *chkbtn_swapfrom;
	GtkWidget *spinbtn_linespc;
	GtkObject *spinbtn_linespc_adj;
	GtkWidget *chkbtn_headspc;
	GtkWidget *chkbtn_disphdr;
} display;

#ifdef USE_GPGME
static struct Privacy {
	GtkWidget *checkbtn_default_encrypt;
	GtkWidget *checkbtn_default_sign;
        GtkWidget *checkbtn_passphrase_grab;
} privacy;
#endif

static struct Interface {
	GtkWidget *checkbtn_emacs;
	GtkWidget *checkbtn_openunread;
	GtkWidget *checkbtn_confonexit;
	GtkWidget *checkbtn_cleanonexit;
	GtkWidget *checkbtn_askonclean;
} interface;

static struct Other {
	GtkWidget *uri_combo;
	GtkWidget *uri_entry;
	GtkWidget *printcmd_entry;
	GtkWidget *exteditor_combo;
	GtkWidget *exteditor_entry;
} other;

static GtkWidget *quote_desc_win;
static GtkWidget *font_sel_win;

static void prefs_common_charset_set_data_from_optmenu(PrefParam *pparam);
static void prefs_common_charset_set_optmenu	      (PrefParam *pparam);

/*
   parameter name, default value, pointer to the prefs variable, data type,
   pointer to the widget pointer,
   pointer to the function for data setting,
   pointer to the function for widget setting
 */

static PrefParam param[] = {
	/* Receive */
	{"use_ext_inc", "FALSE", &prefs_common.use_extinc, P_BOOL,
	 &receive.checkbtn_incext,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"ext_inc_path", DEFAULT_INC_PATH, &prefs_common.extinc_path, P_STRING,
	 &receive.entry_incext,
	 prefs_set_data_from_entry, prefs_set_entry},

	{"inc_local", "TRUE", &prefs_common.inc_local, P_BOOL,
	 &receive.checkbtn_local,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"filter_on_inc_local", "FALSE", &prefs_common.filter_on_inc, P_BOOL,
	 &receive.checkbtn_filter_on_inc,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"spool_path", DEFAULT_SPOOL_PATH, &prefs_common.spool_path, P_STRING,
	 &receive.entry_spool,
	 prefs_set_data_from_entry, prefs_set_entry},

	{"autochk_newmail", "FALSE", &prefs_common.autochk_newmail, P_BOOL,
	 &receive.checkbtn_autochk,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"autochk_interval", "10", &prefs_common.autochk_itv, P_INT,
	 &receive.spinbtn_autochk,
	 prefs_set_data_from_spinbtn, prefs_set_spinbtn},
	{"check_on_startup", "FALSE", &prefs_common.chk_on_startup, P_BOOL,
	 &receive.checkbtn_chkonstartup,
	 prefs_set_data_from_toggle, prefs_set_toggle},

	{"max_news_articles", "300", &prefs_common.max_articles, P_INT,
	 &receive.spinbtn_maxarticle,
	 prefs_set_data_from_spinbtn, prefs_set_spinbtn},

	/* Send */
	{"use_ext_send", "FALSE", &prefs_common.use_extsend, P_BOOL,
	 &send.checkbtn_sendext,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"ext_send_path", NULL, &prefs_common.extsend_path, P_STRING,
	 &send.entry_sendext, prefs_set_data_from_entry, prefs_set_entry},
	{"save_message", "TRUE", &prefs_common.savemsg, P_BOOL,
	 &send.checkbtn_savemsg,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"queue_message", "FALSE", &prefs_common.queue_msg, P_BOOL,
	 &send.checkbtn_queuemsg,
	 prefs_set_data_from_toggle, prefs_set_toggle},

	{"outgoing_charset", CS_AUTO, &prefs_common.outgoing_charset, P_STRING,
	 &send.optmenu_charset,
	 prefs_common_charset_set_data_from_optmenu,
	 prefs_common_charset_set_optmenu},

	/* Compose */
	{"reply_with_quote", "TRUE", &prefs_common.reply_with_quote, P_BOOL,
	 &compose.checkbtn_quote,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"quote_mark", "> ", &prefs_common.quotemark, P_STRING,
	 &compose.entry_quotemark, prefs_set_data_from_entry, prefs_set_entry},
	{"quote_format", "On %d\\n%f wrote:\\n\\n",
	 &prefs_common.quotefmt, P_STRING, &compose.text_quotefmt,
	 prefs_set_data_from_text, prefs_set_text},

	{"auto_signature", "TRUE", &prefs_common.auto_sig, P_BOOL,
	 &compose.checkbtn_autosig,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"signature_separator", "-- ", &prefs_common.sig_sep, P_STRING,
	 &compose.entry_sigsep, prefs_set_data_from_entry, prefs_set_entry},

	{"linewrap_length", "74", &prefs_common.linewrap_len, P_INT,
	 &compose.spinbtn_linewrap,
	 prefs_set_data_from_spinbtn, prefs_set_spinbtn},
	{"linewrap_quotation", "FALSE", &prefs_common.linewrap_quote, P_BOOL,
	 &compose.checkbtn_wrapquote,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"linewrap_before_sending", "FALSE",
	 &prefs_common.linewrap_at_send, P_BOOL,
	 &compose.checkbtn_wrapatsend,
	 prefs_set_data_from_toggle, prefs_set_toggle},

	{"show_ruler", "TRUE", &prefs_common.show_ruler, P_BOOL,
	 NULL, NULL, NULL},

	/* Display */
	{"widget_font", NULL, &prefs_common.widgetfont, P_STRING,
	 NULL, NULL, NULL},
	{"message_font", "-misc-fixed-medium-r-normal--14-*-*-*-*-*-*-*",
	 &prefs_common.messagefont, P_STRING,
	 &display.entry_msgfont,
	 prefs_set_data_from_entry, prefs_set_entry},

	{"display_folder_unread_num", "TRUE",
	 &prefs_common.display_folder_unread, P_BOOL,
	 &display.chkbtn_folder_unread,
	 prefs_set_data_from_toggle, prefs_set_toggle},

	{"enable_color", "TRUE", &prefs_common.enable_color, P_BOOL,
	 &display.chkbtn_enablecol,
	 prefs_set_data_from_toggle, prefs_set_toggle},

	{"quote_color", "0", &prefs_common.quote_col, P_USHORT,
	 NULL, NULL, NULL},
	{"uri_color", "0", &prefs_common.uri_col, P_USHORT,
	 NULL, NULL, NULL},
	{"signature_color", "0", &prefs_common.sig_col, P_USHORT,
	 NULL, NULL, NULL},
	{"toolbar_style", "3", &prefs_common.toolbar_style, P_ENUM,
	 NULL, NULL, NULL},
	{"show_statusbar", "TRUE", &prefs_common.show_statusbar, P_BOOL,
	 NULL, NULL, NULL},

	{"translate_header", "TRUE", &prefs_common.trans_hdr, P_BOOL,
	 &display.chkbtn_transhdr,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"enable_thread", "TRUE", &prefs_common.enable_thread, P_BOOL,
	 NULL, NULL, NULL},
	{"convert_mb_alnum", "FALSE", &prefs_common.conv_mb_alnum, P_BOOL,
	 &display.chkbtn_mbalnum,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"enable_swap_from", "TRUE", &prefs_common.swap_from, P_BOOL,
	 &display.chkbtn_swapfrom,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"line_space", "2", &prefs_common.line_space, P_INT,
	 &display.spinbtn_linespc,
	 prefs_set_data_from_spinbtn, prefs_set_spinbtn},
	{"enable_head_space", "FALSE", &prefs_common.head_space, P_BOOL,
	 &display.chkbtn_headspc,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"display_header", "TRUE", &prefs_common.display_header, P_BOOL,
	 &display.chkbtn_disphdr,
	 prefs_set_data_from_toggle, prefs_set_toggle},

	/* MIME viewer */
	{"mime_image_viewer", "display %s",
	 &prefs_common.mime_image_viewer, P_STRING, NULL, NULL, NULL},
	{"mime_audio_player", "play %s",
	 &prefs_common.mime_audio_player, P_STRING, NULL, NULL, NULL},

	{"show_mark", "TRUE", &prefs_common.show_mark, P_BOOL,
	 NULL, NULL, NULL},
	{"show_unread", "TRUE", &prefs_common.show_unread, P_BOOL,
	 NULL, NULL, NULL},
	{"show_mime", "TRUE", &prefs_common.show_mime, P_BOOL,
	 NULL, NULL, NULL},
	{"show_number", "TRUE", &prefs_common.show_number, P_BOOL,
	 NULL, NULL, NULL},
	{"show_size", "FALSE", &prefs_common.show_size, P_BOOL,
	 NULL, NULL, NULL},
	{"show_date", "TRUE", &prefs_common.show_date, P_BOOL,
	 NULL, NULL, NULL},
	{"show_from", "TRUE", &prefs_common.show_from, P_BOOL,
	 NULL, NULL, NULL},
	{"show_subject", "TRUE", &prefs_common.show_subject, P_BOOL,
	 NULL, NULL, NULL},

	/* Widget size */
	{"folderview_width", "179", &prefs_common.folderview_width, P_INT,
	 NULL, NULL, NULL},
	{"folderview_height", "600", &prefs_common.folderview_height, P_INT,
	 NULL, NULL, NULL},
	{"folder_col_folder", "150", &prefs_common.folder_col_folder, P_INT,
	 NULL, NULL, NULL},
	{"folder_col_new", "32", &prefs_common.folder_col_new, P_INT,
	 NULL, NULL, NULL},
	{"folder_col_unread", "32", &prefs_common.folder_col_unread, P_INT,
	 NULL, NULL, NULL},
	{"folder_col_total", "32", &prefs_common.folder_col_total, P_INT,
	 NULL, NULL, NULL},
	{"summaryview_width", "600", &prefs_common.summaryview_width, P_INT,
	 NULL, NULL, NULL},
	{"summaryview_height", "173", &prefs_common.summaryview_height, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_mark", "10", &prefs_common.summary_col_mark, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_unread", "10", &prefs_common.summary_col_unread, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_mime", "10", &prefs_common.summary_col_mime, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_number", "40", &prefs_common.summary_col_number, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_size", "48", &prefs_common.summary_col_size, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_date", "120", &prefs_common.summary_col_date, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_from", "140", &prefs_common.summary_col_from, P_INT,
	 NULL, NULL, NULL},
	{"summary_col_subject", "200", &prefs_common.summary_col_subject,
	 P_INT, NULL, NULL, NULL},
	{"mainview_x", "64", &prefs_common.mainview_x, P_INT,
	 NULL, NULL, NULL},
	{"mainview_y", "64", &prefs_common.mainview_y, P_INT,
	 NULL, NULL, NULL},
	{"mainview_width", "600", &prefs_common.mainview_width, P_INT,
	 NULL, NULL, NULL},
	{"mainview_height", "600", &prefs_common.mainview_height, P_INT,
	 NULL, NULL, NULL},
	{"mainwin_x", "64", &prefs_common.mainwin_x, P_INT,
	 NULL, NULL, NULL},
	{"mainwin_y", "64", &prefs_common.mainwin_y, P_INT,
	 NULL, NULL, NULL},
	{"mainwin_width", "800", &prefs_common.mainwin_width, P_INT,
	 NULL, NULL, NULL},
	{"mainwin_height", "600", &prefs_common.mainwin_height, P_INT,
	 NULL, NULL, NULL},
	{"messagewin_width", "600", &prefs_common.msgwin_width, P_INT,
	 NULL, NULL, NULL},
	{"messagewin_height", "540", &prefs_common.msgwin_height, P_INT,
	 NULL, NULL, NULL},
	{"compose_width", "600", &prefs_common.compose_width, P_INT,
	 NULL, NULL, NULL},
	{"compose_height", "560", &prefs_common.compose_height, P_INT,
	 NULL, NULL, NULL},

#ifdef USE_GPGME
        /* Privacy */
	{"default_encrypt", "FALSE", &prefs_common.default_encrypt, P_BOOL,
	 &privacy.checkbtn_default_encrypt,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"default_sign", "FALSE", &prefs_common.default_sign, P_BOOL,
	 &privacy.checkbtn_default_sign,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"passphrase_grab", "FALSE", &prefs_common.passphrase_grab, P_BOOL,
	 &privacy.checkbtn_passphrase_grab,
	 prefs_set_data_from_toggle, prefs_set_toggle},
#endif /*USE_GPGME*/

	/* Interface */
	{"separate_folder", "FALSE", &prefs_common.sep_folder, P_BOOL,
	 NULL, NULL, NULL},
	{"separate_message", "FALSE", &prefs_common.sep_msg, P_BOOL,
	 NULL, NULL, NULL},

	{"emulate_emacs", "FALSE", &prefs_common.emulate_emacs, P_BOOL,
	 &interface.checkbtn_emacs,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"open_unread_on_enter", "FALSE", &prefs_common.open_unread_on_enter,
	 P_BOOL, &interface.checkbtn_openunread,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"confirm_on_exit", "TRUE", &prefs_common.confirm_on_exit, P_BOOL,
	 &interface.checkbtn_confonexit,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"clean_trash_on_exit", "FALSE", &prefs_common.clean_on_exit, P_BOOL,
	 &interface.checkbtn_cleanonexit,
	 prefs_set_data_from_toggle, prefs_set_toggle},
	{"ask_on_cleaning", "TRUE", &prefs_common.ask_on_clean, P_BOOL,
	 &interface.checkbtn_askonclean,
	 prefs_set_data_from_toggle, prefs_set_toggle},

	/* Other */
	{"uri_open_command", "netscape -remote 'openURL(%s,raise)'",
	 &prefs_common.uri_cmd, P_STRING,
	 &other.uri_entry, prefs_set_data_from_entry, prefs_set_entry},
	{"print_command", "lpr %s", &prefs_common.print_cmd, P_STRING,
	 &other.printcmd_entry, prefs_set_data_from_entry, prefs_set_entry},
	{"ext_editor_command", "gedit %s",
	 &prefs_common.ext_editor_cmd, P_STRING,
	 &other.exteditor_entry, prefs_set_data_from_entry, prefs_set_entry},

	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
};

#define VSPACING	12
#define VSPACING_NARROW	6
#define VBOX_BORDER	16
#define PREFSBUFSIZE	1024

/* widget creating functions */
static void prefs_common_create		(void);
static void prefs_receive_create	(void);
static void prefs_send_create		(void);
static void prefs_compose_create	(void);
static void prefs_filter_create		(void);
static void prefs_display_create	(void);
#ifdef USE_GPGME
  static void prefs_privacy_create	(void);
#endif
static void prefs_interface_create	(void);
static void prefs_other_create		(void);

static void prefs_quote_description	   (void);
static void prefs_quote_description_create (void);

static void prefs_filter_read_config	(void);
static void prefs_filter_write_config	(void);
static void prefs_filter_set_dialog	(void);
static void prefs_filter_set_list	(void);
static gint prefs_filter_clist_set_row	(gint	 row);

/* callback functions */
static void prefs_filter_select_dest_cb	(void);
static void prefs_filter_register_cb	(void);
static void prefs_filter_substitute_cb	(void);
static void prefs_filter_delete_cb	(void);
static void prefs_filter_up		(void);
static void prefs_filter_down		(void);
static void prefs_filter_select		(GtkCList	*clist,
					 gint		 row,
					 gint		 column,
					 GdkEvent	*event);

static void prefs_filter_dest_radio_button_toggled	(void);
static void prefs_filter_notrecv_radio_button_toggled	(void);

/* functions for setting items of SummaryView */
static void prefs_summary_display_item_dialog_create
					(gboolean	*cancelled);
static void display_item_ok		(GtkWidget	*widget,
					 gboolean	*cancelled);
static void display_item_cancel		(GtkWidget	*widget,
					 gboolean	*cancelled);
static gint display_item_delete_event	(GtkWidget	*widget,
					 GdkEventAny	*event,
					 gboolean	*cancelled);
static void display_item_key_pressed	(GtkWidget	*widget,
					 GdkEventKey	*event,
					 gboolean	*cancelled);

static void prefs_font_select			(GtkButton	*button);
static void prefs_font_selection_key_pressed	(GtkWidget	*widget,
						 GdkEventKey	*event,
						 gpointer	 data);
static void prefs_font_selection_ok		(GtkButton	*button);

static void prefs_common_key_pressed	(GtkWidget	*widget,
					 GdkEventKey	*event,
					 gpointer	 data);
static void prefs_common_ok		(GtkButton	*button);
static void prefs_common_apply		(GtkButton	*button);

void prefs_common_read_config(void)
{
	prefs_read_config(param, "Common", COMMON_RC);
	prefs_filter_read_config();
}

void prefs_common_save_config(void)
{
	prefs_save_config(param, "Common", COMMON_RC);
	prefs_filter_write_config();
}

void prefs_common_open(void)
{
	if (!dialog.window) {
		prefs_common_create();
	}

	manage_window_set_transient(GTK_WINDOW(dialog.window));
	gtk_notebook_set_page(GTK_NOTEBOOK(dialog.notebook), 0);
	gtk_widget_grab_focus(dialog.ok_btn);

	prefs_set_dialog(param);
	prefs_filter_set_dialog();

	gtk_widget_show(dialog.window);
}

static void prefs_common_create(void)
{
	GtkStyle *style;
	gint page = 0;

	debug_print(_("Creating common preferences window...\n"));

	prefs_dialog_create(&dialog);
	gtk_window_set_title (GTK_WINDOW(dialog.window),
			      _("Common Preferences"));
	gtk_signal_connect (GTK_OBJECT(dialog.window), "delete_event",
			    GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete), NULL);
	gtk_signal_connect (GTK_OBJECT(dialog.window), "key_press_event",
			    GTK_SIGNAL_FUNC(prefs_common_key_pressed), NULL);
	gtk_signal_connect (GTK_OBJECT(dialog.window), "focus_in_event",
			    GTK_SIGNAL_FUNC(manage_window_focus_in), NULL);
	gtk_signal_connect (GTK_OBJECT(dialog.window), "focus_out_event",
			    GTK_SIGNAL_FUNC(manage_window_focus_out), NULL);
	gtk_signal_connect (GTK_OBJECT(dialog.ok_btn), "clicked",
			    GTK_SIGNAL_FUNC(prefs_common_ok), NULL);
	gtk_signal_connect (GTK_OBJECT(dialog.apply_btn), "clicked",
			    GTK_SIGNAL_FUNC(prefs_common_apply), NULL);
	gtk_signal_connect_object (GTK_OBJECT(dialog.cancel_btn), "clicked",
				   GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete),
				   GTK_OBJECT(dialog.window));

	/* create all widgets on notebook */
	prefs_receive_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Receive"),   page++);
	prefs_send_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Send"),      page++);
	prefs_compose_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Compose"),   page++);
	prefs_display_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Display"),   page++);
	prefs_filter_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Filtering"), page++);
#ifdef USE_GPGME
	prefs_privacy_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Privacy"), page++);
#endif
	prefs_interface_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Interface"), page++);
	prefs_other_create();
	SET_NOTEBOOK_LABEL(dialog.notebook, _("Other"),     page++);

	gtk_widget_show_all(dialog.window);

	style = gtk_widget_get_style(dialog.window);
	if (style && style->font) {
		gint width, height;

		width = gtkut_get_font_width(style->font);
		height = gtkut_get_font_height(style->font);
		gtk_widget_set_usize
			(dialog.notebook,
			 width * 58 + VBOX_BORDER * 2 + 24,
			 height * 10 + VBOX_BORDER * 2 + VSPACING * 15 + 64);
	}
}

static void prefs_receive_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *frame_incext;
	GtkWidget *checkbtn_incext;
	GtkWidget *hbox;
	GtkWidget *label_incext;
	GtkWidget *entry_incext;
	GtkWidget *button_incext;

	GtkWidget *frame_spool;
	GtkWidget *checkbtn_local;
	GtkWidget *checkbtn_filter_on_inc;
	GtkWidget *label_spool;
	GtkWidget *entry_spool;

	GtkWidget *hbox_autochk;
	GtkWidget *checkbtn_autochk;
	GtkWidget *label_autochk1;
	GtkObject *spinbtn_autochk_adj;
	GtkWidget *spinbtn_autochk;
	GtkWidget *label_autochk2;
	GtkWidget *checkbtn_chkonstartup;

	GtkWidget *frame_news;
	GtkWidget *label_maxarticle;
	GtkWidget *spinbtn_maxarticle;
	GtkObject *spinbtn_maxarticle_adj;

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	PACK_FRAME(vbox1, frame_incext, _("External program"));

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (frame_incext), vbox2);
	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);

	/* Use of external incorporation program */
	PACK_CHECK_BUTTON (vbox2, checkbtn_incext,
			   _("Use external program for incorporation"));

	hbox = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
	SET_TOGGLE_SENSITIVITY (checkbtn_incext, hbox);

	label_incext = gtk_label_new (_("Program path"));
	gtk_widget_show (label_incext);
	gtk_box_pack_start (GTK_BOX (hbox), label_incext, FALSE, FALSE, 0);

	entry_incext = gtk_entry_new ();
	gtk_widget_show (entry_incext);
	gtk_box_pack_start (GTK_BOX (hbox), entry_incext, TRUE, TRUE, 0);

	button_incext = gtk_button_new_with_label ("... ");
	gtk_widget_show (button_incext);
	gtk_box_pack_start (GTK_BOX (hbox), button_incext, FALSE, FALSE, 0);

	PACK_FRAME(vbox1, frame_spool, _("Local spool"));

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (frame_spool), vbox2);
	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);

	hbox = gtk_hbox_new (FALSE, 32);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);

	PACK_CHECK_BUTTON (hbox, checkbtn_local, _("Incorporate from spool"));
	PACK_CHECK_BUTTON (hbox, checkbtn_filter_on_inc,
			   _("Filter on incorporation"));
	SET_TOGGLE_SENSITIVITY (checkbtn_local, checkbtn_filter_on_inc);

	hbox = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
	SET_TOGGLE_SENSITIVITY (checkbtn_local, hbox);

	label_spool = gtk_label_new (_("Spool directory"));
	gtk_widget_show (label_spool);
	gtk_box_pack_start (GTK_BOX (hbox), label_spool, FALSE, FALSE, 0);

	entry_spool = gtk_entry_new ();
	gtk_widget_show (entry_spool);
	gtk_box_pack_start (GTK_BOX (hbox), entry_spool, TRUE, TRUE, 0);

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);

	/* Auto-checking */
	hbox_autochk = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox_autochk);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox_autochk, FALSE, FALSE, 0);

	gtk_widget_set_sensitive(hbox_autochk, FALSE);

	PACK_CHECK_BUTTON (hbox_autochk, checkbtn_autochk,
			   _("Auto-check new mail"));

	label_autochk1 = gtk_label_new (_("each"));
	gtk_widget_show (label_autochk1);
	gtk_box_pack_start (GTK_BOX (hbox_autochk), label_autochk1, FALSE, FALSE, 0);

	spinbtn_autochk_adj = gtk_adjustment_new (5, 1, 100, 1, 10, 10);
	spinbtn_autochk = gtk_spin_button_new
		(GTK_ADJUSTMENT (spinbtn_autochk_adj), 1, 0);
	gtk_widget_show (spinbtn_autochk);
	gtk_box_pack_start (GTK_BOX (hbox_autochk), spinbtn_autochk, FALSE, FALSE, 0);
	gtk_widget_set_usize (spinbtn_autochk, 64, -1);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_autochk), TRUE);

	label_autochk2 = gtk_label_new (_("minute(s)"));
	gtk_widget_show (label_autochk2);
	gtk_box_pack_start (GTK_BOX (hbox_autochk), label_autochk2, FALSE, FALSE, 0);

	SET_TOGGLE_SENSITIVITY(checkbtn_autochk, label_autochk1);
	SET_TOGGLE_SENSITIVITY(checkbtn_autochk, spinbtn_autochk);
	SET_TOGGLE_SENSITIVITY(checkbtn_autochk, label_autochk2);

	PACK_CHECK_BUTTON (vbox2, checkbtn_chkonstartup,
			   _("Check new mail on startup"));

	PACK_FRAME(vbox1, frame_news, _("News"));

	hbox = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox);
	gtk_container_add (GTK_CONTAINER (frame_news), hbox);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);

	label_maxarticle = gtk_label_new
		(_("Maximum article number to download\n"
		   "(unlimited if 0 is specified)"));
	gtk_widget_show (label_maxarticle);
	gtk_box_pack_start (GTK_BOX (hbox), label_maxarticle, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label_maxarticle), GTK_JUSTIFY_LEFT);

	spinbtn_maxarticle_adj =
		gtk_adjustment_new (300, 0, 10000, 10, 100, 100);
	spinbtn_maxarticle = gtk_spin_button_new
		(GTK_ADJUSTMENT (spinbtn_maxarticle_adj), 10, 0);
	gtk_widget_show (spinbtn_maxarticle);
	gtk_box_pack_start (GTK_BOX (hbox), spinbtn_maxarticle,
			    FALSE, FALSE, 0);
	gtk_widget_set_usize (spinbtn_maxarticle, 64, -1);
	gtk_spin_button_set_numeric
		(GTK_SPIN_BUTTON (spinbtn_maxarticle), TRUE);

	receive.checkbtn_incext = checkbtn_incext;
	receive.entry_incext    = entry_incext;
	receive.button_incext   = button_incext;

	receive.checkbtn_local         = checkbtn_local;
	receive.checkbtn_filter_on_inc = checkbtn_filter_on_inc;
	receive.entry_spool            = entry_spool;

	receive.checkbtn_autochk    = checkbtn_autochk;
	receive.spinbtn_autochk     = spinbtn_autochk;
	receive.spinbtn_autochk_adj = spinbtn_autochk_adj;

	receive.checkbtn_chkonstartup = checkbtn_chkonstartup;

	receive.spinbtn_maxarticle     = spinbtn_maxarticle;
	receive.spinbtn_maxarticle_adj = spinbtn_maxarticle_adj;
}

static void prefs_send_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *frame_sendext;
	GtkWidget *vbox_sendext;
	GtkWidget *checkbtn_sendext;
	GtkWidget *hbox1;
	GtkWidget *label_sendext;
	GtkWidget *entry_sendext;
	GtkWidget *button_sendext;
	GtkWidget *checkbtn_savemsg;
	GtkWidget *checkbtn_queuemsg;
	GtkWidget *label_outcharset;
	GtkWidget *optmenu;
	GtkWidget *optmenu_menu;
	GtkWidget *menuitem;

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	PACK_FRAME(vbox1, frame_sendext, _("External program"));

	gtk_widget_set_sensitive(frame_sendext, FALSE);

	vbox_sendext = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox_sendext);
	gtk_container_add (GTK_CONTAINER (frame_sendext), vbox_sendext);
	gtk_container_set_border_width (GTK_CONTAINER (vbox_sendext), 8);

	PACK_CHECK_BUTTON (vbox_sendext, checkbtn_sendext,
			   _("Use external program for sending"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox_sendext), hbox1, FALSE, FALSE, 0);
	SET_TOGGLE_SENSITIVITY(checkbtn_sendext, hbox1);

	label_sendext = gtk_label_new (_("Program path"));
	gtk_widget_show (label_sendext);
	gtk_box_pack_start (GTK_BOX (hbox1), label_sendext, FALSE, FALSE, 0);

	entry_sendext = gtk_entry_new ();
	gtk_widget_show (entry_sendext);
	gtk_box_pack_start (GTK_BOX (hbox1), entry_sendext, TRUE, TRUE, 0);

	button_sendext = gtk_button_new_with_label ("... ");
	gtk_widget_show (button_sendext);
	gtk_box_pack_start (GTK_BOX (hbox1), button_sendext, FALSE, FALSE, 0);

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);

	PACK_CHECK_BUTTON (vbox2, checkbtn_savemsg,
			   _("Save sent message to outbox"));
	PACK_CHECK_BUTTON (vbox2, checkbtn_queuemsg,
			   _("Queue message that failed to send"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

	label_outcharset = gtk_label_new (_("Outgoing codeset"));
	gtk_widget_show (label_outcharset);
	gtk_box_pack_start (GTK_BOX (hbox1), label_outcharset, FALSE, FALSE, 0);

	optmenu = gtk_option_menu_new ();
	gtk_widget_show (optmenu);
	gtk_box_pack_start(GTK_BOX (hbox1), optmenu, FALSE, FALSE, 0);

	optmenu_menu = gtk_menu_new ();

#define SET_MENUITEM(str, charset) \
{ \
	MENUITEM_ADD(optmenu_menu, menuitem, str, charset); \
}

	SET_MENUITEM(_("Automatic"),			 CS_AUTO);
	SET_MENUITEM(_("7bit ascii (US-ASCII)"),	 CS_US_ASCII);
#if HAVE_LIBJCONV
	SET_MENUITEM(_("Unicode (UTF-8)"),		 CS_UTF_8);
#endif
	SET_MENUITEM(_("Western European (ISO-8859-1)"), CS_ISO_8859_1);
#if HAVE_LIBJCONV
	SET_MENUITEM(_("Central European (ISO-8859-2)"), CS_ISO_8859_2);
	SET_MENUITEM(_("Baltic (ISO-8859-13)"),		 CS_ISO_8859_13);
	SET_MENUITEM(_("Baltic (ISO-8859-4)"),		 CS_ISO_8859_4);
	SET_MENUITEM(_("Greek (ISO-8859-7)"),		 CS_ISO_8859_7);
	SET_MENUITEM(_("Turkish (ISO-8859-9)"),		 CS_ISO_8859_9);
	SET_MENUITEM(_("Cyrillic (ISO-8859-5)"),	 CS_ISO_8859_5);
	SET_MENUITEM(_("Cyrillic (KOI8-R)"),		 CS_KOI8_R);
	SET_MENUITEM(_("Cyrillic (Windows-1251)"),	 CS_CP1251);
	SET_MENUITEM(_("Cyrillic (KOI8-U)"),		 CS_KOI8_U);
#endif
	SET_MENUITEM(_("Japanese (ISO-2022-JP)"),	 CS_ISO_2022_JP);
	SET_MENUITEM(_("Japanese (EUC-JP)"),		 CS_EUC_JP);
	SET_MENUITEM(_("Japanese (Shift_JIS)"),		 CS_SHIFT_JIS);
#if HAVE_LIBJCONV
	SET_MENUITEM(_("Simplified Chinese (GB2312)"),	 CS_GB2312);
	SET_MENUITEM(_("Traditional Chinese (Big5)"),	 CS_BIG5);
	//SET_MENUITEM(_("Traditional Chinese (EUC-TW)"),CS_EUC_TW);
	//SET_MENUITEM(_("Chinese (ISO-2022-CN)"),	 CS_ISO_2022_CN);
	SET_MENUITEM(_("Korean (EUC-KR)"),		 CS_EUC_KR);
#endif

	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);

	send.checkbtn_sendext = checkbtn_sendext;
	send.entry_sendext    = entry_sendext;
	send.button_sendext   = button_sendext;

	send.checkbtn_savemsg  = checkbtn_savemsg;
	send.checkbtn_queuemsg = checkbtn_queuemsg;

	send.optmenu_charset = optmenu;
}

static void prefs_compose_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *frame_quote;
	GtkWidget *vbox_quote;
	GtkWidget *checkbtn_quote;
	GtkWidget *hbox1;
	GtkWidget *label_quotemark;
	GtkWidget *entry_quotemark;
	GtkWidget *hbox2;
	GtkWidget *label_quotefmt;
	GtkWidget *btn_quotedesc;
	GtkWidget *scrolledwin_quotefmt;
	GtkWidget *text_quotefmt;

	GtkWidget *frame_sig;
	GtkWidget *vbox_sig;
	GtkWidget *checkbtn_autosig;
	GtkWidget *label_sigsep;
	GtkWidget *entry_sigsep;

	GtkWidget *vbox_linewrap;
	GtkWidget *hbox3;
	GtkWidget *hbox4;
	GtkWidget *label_linewrap;
	GtkObject *spinbtn_linewrap_adj;
	GtkWidget *spinbtn_linewrap;
	GtkWidget *checkbtn_wrapquote;
	GtkWidget *checkbtn_wrapatsend;

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	PACK_FRAME(vbox1, frame_quote, _("Quotation"));

	vbox_quote = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox_quote);
	gtk_container_add (GTK_CONTAINER (frame_quote), vbox_quote);
	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);

	PACK_CHECK_BUTTON (vbox_quote, checkbtn_quote,
			   _("Quote message when replying"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, TRUE, TRUE, 0);

	label_quotemark = gtk_label_new (_("Quotation mark"));
	gtk_widget_show (label_quotemark);
	gtk_box_pack_start (GTK_BOX (hbox1), label_quotemark, FALSE, FALSE, 0);

	entry_quotemark = gtk_entry_new ();
	gtk_widget_show (entry_quotemark);
	gtk_box_pack_start (GTK_BOX (hbox1), entry_quotemark, FALSE, FALSE, 0);
	gtk_widget_set_usize (entry_quotemark, 64, -1);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox2, TRUE, TRUE, 0);

	label_quotefmt = gtk_label_new (_("Quotation format:"));
	gtk_widget_show (label_quotefmt);
	gtk_box_pack_start (GTK_BOX (hbox2), label_quotefmt, FALSE, FALSE, 0);

	btn_quotedesc =
		gtk_button_new_with_label (_(" Description of symbols "));
	gtk_widget_show (btn_quotedesc);
	gtk_box_pack_end (GTK_BOX (hbox2), btn_quotedesc, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(btn_quotedesc), "clicked",
			   GTK_SIGNAL_FUNC(prefs_quote_description), NULL);

	scrolledwin_quotefmt = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwin_quotefmt);
	gtk_box_pack_start (GTK_BOX (vbox_quote), scrolledwin_quotefmt, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy
		(GTK_SCROLLED_WINDOW (scrolledwin_quotefmt),
		 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	text_quotefmt = gtk_text_new (NULL, NULL);
	gtk_widget_show (text_quotefmt);
	gtk_container_add(GTK_CONTAINER(scrolledwin_quotefmt), text_quotefmt);
	gtk_text_set_editable (GTK_TEXT (text_quotefmt), TRUE);
	gtk_widget_set_usize(text_quotefmt, -1, 60);

	PACK_FRAME(vbox1, frame_sig, _("Signature"));

	vbox_sig = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox_sig);
	gtk_container_add (GTK_CONTAINER (frame_sig), vbox_sig);
	gtk_container_set_border_width (GTK_CONTAINER (vbox_sig), 8);

	PACK_CHECK_BUTTON (vbox_sig, checkbtn_autosig,
			   _("Insert signature automatically"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox_sig), hbox1, TRUE, TRUE, 0);

	label_sigsep = gtk_label_new (_("Signature separator"));
	gtk_widget_show (label_sigsep);
	gtk_box_pack_start (GTK_BOX (hbox1), label_sigsep, FALSE, FALSE, 0);

	entry_sigsep = gtk_entry_new ();
	gtk_widget_show (entry_sigsep);
	gtk_box_pack_start (GTK_BOX (hbox1), entry_sigsep, FALSE, FALSE, 0);
	gtk_widget_set_usize (entry_sigsep, 64, -1);

	/* line-wrapping */
	vbox_linewrap = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox_linewrap);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox_linewrap, FALSE, FALSE, 0);

	hbox3 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox3);
	gtk_box_pack_start (GTK_BOX (vbox_linewrap), hbox3, FALSE, FALSE, 0);

	label_linewrap = gtk_label_new (_("Wrap messages at"));
	gtk_widget_show (label_linewrap);
	gtk_box_pack_start (GTK_BOX (hbox3), label_linewrap, FALSE, FALSE, 0);

	spinbtn_linewrap_adj = gtk_adjustment_new (72, 20, 1024, 1, 10, 10);
	spinbtn_linewrap = gtk_spin_button_new
		(GTK_ADJUSTMENT (spinbtn_linewrap_adj), 1, 0);
	gtk_widget_show (spinbtn_linewrap);
	gtk_box_pack_start (GTK_BOX (hbox3), spinbtn_linewrap, FALSE, FALSE, 0);
	gtk_widget_set_usize (spinbtn_linewrap, 64, -1);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_linewrap), TRUE);

	label_linewrap = gtk_label_new (_("characters"));
	gtk_widget_show (label_linewrap);
	gtk_box_pack_start (GTK_BOX (hbox3), label_linewrap, FALSE, FALSE, 0);

	hbox4 = gtk_hbox_new (FALSE, 32);
	gtk_widget_show (hbox4);
	gtk_box_pack_start (GTK_BOX (vbox_linewrap), hbox4, FALSE, FALSE, 0);

	PACK_CHECK_BUTTON (hbox4, checkbtn_wrapquote, _("Wrap quotation"));
	PACK_CHECK_BUTTON
		(hbox4, checkbtn_wrapatsend, _("Wrap before sending"));

	compose.checkbtn_quote   = checkbtn_quote;
	compose.entry_quotemark  = entry_quotemark;
	compose.text_quotefmt    = text_quotefmt;
	compose.checkbtn_autosig = checkbtn_autosig;
	compose.entry_sigsep     = entry_sigsep;

	compose.spinbtn_linewrap     = spinbtn_linewrap;
	compose.spinbtn_linewrap_adj = spinbtn_linewrap_adj;
	compose.checkbtn_wrapquote   = checkbtn_wrapquote;
	compose.checkbtn_wrapatsend  = checkbtn_wrapatsend;
}

static void prefs_filter_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *table1;
	GtkWidget *op_label;
	GtkWidget *op_combo;
	GtkWidget *op_entry;
	GtkWidget *hdr_label;
	GtkWidget *hdr_combo1;
	GtkWidget *hdr_combo2;
	GtkWidget *key_label;
	GtkWidget *key_entry1;
	GtkWidget *key_entry2;
	GtkWidget *pred_label;
	GtkWidget *pred_combo1;
	GtkWidget *pred_entry1;
	GtkWidget *pred_combo2;
	GtkWidget *pred_entry2;

	GtkWidget *vbox2;
	GtkWidget *dest_hbox;
	GtkWidget *dest_entry;
	GtkWidget *destsel_btn;
	GtkWidget *dest_radiobtn;
	GtkWidget *notrecv_radiobtn;
	GSList *recv_group = NULL;

	GtkWidget *regex_chkbtn;

	GtkWidget *reg_hbox;
	GtkWidget *btn_hbox;
	GtkWidget *arrow;
	GtkWidget *reg_btn;
	GtkWidget *subst_btn;
	GtkWidget *del_btn;

	GtkWidget *cond_hbox;
	GtkWidget *cond_scrolledwin;
	GtkWidget *cond_clist;

	GtkWidget *btn_vbox;
	GtkWidget *up_btn;
	GtkWidget *down_btn;

	gchar *title[] = {_("Registered rules")};

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	table1 = gtk_table_new (3, 4, FALSE);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (vbox1), table1, FALSE, FALSE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 8);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 8);

	op_label = gtk_label_new (_("Operator"));
	gtk_widget_show (op_label);
	gtk_table_attach (GTK_TABLE (table1), op_label, 0, 1, 0, 1,
			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
			  0, 0, 0);
	gtk_misc_set_alignment (GTK_MISC (op_label), 0, 0.5);

	op_combo = gtk_combo_new ();
	gtk_widget_show (op_combo);
	gtk_table_attach (GTK_TABLE (table1), op_combo, 0, 1, 2, 3,
			  0, 0, 0, 0);
	gtk_widget_set_usize (op_combo, 52, -1);
	gtkut_combo_set_items (GTK_COMBO (op_combo), "and", "or", NULL);

	op_entry = GTK_COMBO (op_combo)->entry;
	gtk_entry_set_editable (GTK_ENTRY (op_entry), FALSE);

	hdr_label = gtk_label_new (_("Header"));
	gtk_widget_show (hdr_label);
	gtk_table_attach (GTK_TABLE (table1), hdr_label, 1, 2, 0, 1,
			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
			  0, 0, 0);
	gtk_misc_set_alignment (GTK_MISC (hdr_label), 0, 0.5);

	hdr_combo1 = gtk_combo_new ();
	gtk_widget_show (hdr_combo1);
	gtk_table_attach (GTK_TABLE (table1), hdr_combo1, 1, 2, 1, 2,
			  0, 0, 0, 0);
	gtk_widget_set_usize (hdr_combo1, 96, -1);
	gtkut_combo_set_items (GTK_COMBO (hdr_combo1),
			       "Subject", "From", "To", "Cc", "Reply-To",
			       "Sender", "X-ML-Name", "X-List", "X-Sequence",
			       "X-Mailer",
			       NULL);

	hdr_combo2 = gtk_combo_new ();
	gtk_widget_show (hdr_combo2);
	gtk_table_attach (GTK_TABLE (table1), hdr_combo2, 1, 2, 2, 3,
			  0, 0, 0, 0);
	gtk_widget_set_usize (hdr_combo2, 96, -1);
	gtkut_combo_set_items (GTK_COMBO (hdr_combo2), _("(none)"),
			       "Subject", "From", "To", "Cc", "Reply-To",
			       "Sender", "X-ML-Name", "X-List", "X-Sequence",
			       "X-Mailer",
			       NULL);

	key_label = gtk_label_new (_("Keyword"));
	gtk_widget_show (key_label);
	gtk_table_attach (GTK_TABLE (table1), key_label, 2, 3, 0, 1,
			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
			  0, 0, 0);
	gtk_misc_set_alignment (GTK_MISC (key_label), 0, 0.5);

	key_entry1 = gtk_entry_new ();
	gtk_widget_show (key_entry1);
	gtk_table_attach (GTK_TABLE (table1), key_entry1, 2, 3, 1, 2,
			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
			  0, 0, 0);

	key_entry2 = gtk_entry_new ();
	gtk_widget_show (key_entry2);
	gtk_table_attach (GTK_TABLE (table1), key_entry2, 2, 3, 2, 3,
			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
			  0, 0, 0);

	pred_label = gtk_label_new (_("Predicate"));
	gtk_widget_show (pred_label);
	gtk_table_attach (GTK_TABLE (table1), pred_label, 3, 4, 0, 1,
			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
			  0, 0, 0);
	gtk_misc_set_alignment (GTK_MISC (pred_label), 0, 0.5);

	pred_combo1 = gtk_combo_new ();
	gtk_widget_show (pred_combo1);
	gtk_table_attach (GTK_TABLE (table1), pred_combo1, 3, 4, 1, 2,
			  0, 0, 0, 0);
	gtk_widget_set_usize (pred_combo1, 92, -1);
	gtkut_combo_set_items (GTK_COMBO (pred_combo1),
			       _("contains"), _("not contain"), NULL);

	pred_entry1 = GTK_COMBO (pred_combo1)->entry;
	gtk_entry_set_editable (GTK_ENTRY (pred_entry1), FALSE);

	pred_combo2 = gtk_combo_new ();
	gtk_widget_show (pred_combo2);
	gtk_table_attach (GTK_TABLE (table1), pred_combo2, 3, 4, 2, 3,
			  0, 0, 0, 0);
	gtk_widget_set_usize (pred_combo2, 92, -1);
	gtkut_combo_set_items (GTK_COMBO (pred_combo2),
			       _("contains"), _("not contain"), NULL);

	pred_entry2 = GTK_COMBO (pred_combo2)->entry;
	gtk_entry_set_editable (GTK_ENTRY (pred_entry2), FALSE);

	/* destination */

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, TRUE, 0);

	dest_hbox = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (dest_hbox);
	gtk_box_pack_start (GTK_BOX (vbox2), dest_hbox, FALSE, TRUE, 0);

	dest_radiobtn =
		gtk_radio_button_new_with_label (recv_group, _("Destination"));
	recv_group = gtk_radio_button_group (GTK_RADIO_BUTTON (dest_radiobtn));
	gtk_widget_show (dest_radiobtn);
	gtk_box_pack_start (GTK_BOX (dest_hbox), dest_radiobtn,
			    FALSE, FALSE, 0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dest_radiobtn), TRUE);
	gtk_signal_connect
		(GTK_OBJECT (dest_radiobtn), "toggled",
		 GTK_SIGNAL_FUNC (prefs_filter_dest_radio_button_toggled),
		 NULL);

	dest_entry = gtk_entry_new ();
	gtk_widget_show (dest_entry);
	gtk_box_pack_start (GTK_BOX (dest_hbox), dest_entry, TRUE, TRUE, 0);
	gtk_entry_set_editable (GTK_ENTRY (dest_entry), FALSE);

	destsel_btn = gtk_button_new_with_label (_(" Select... "));
	gtk_widget_show (destsel_btn);
	gtk_box_pack_start (GTK_BOX (dest_hbox), destsel_btn, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (destsel_btn), "clicked",
			    GTK_SIGNAL_FUNC (prefs_filter_select_dest_cb),
			    NULL);

	PACK_CHECK_BUTTON (dest_hbox, regex_chkbtn, _("Use regex"));
	gtk_widget_set_sensitive(regex_chkbtn, FALSE);

	notrecv_radiobtn = gtk_radio_button_new_with_label
		(recv_group, _("Don't receive"));
	recv_group = gtk_radio_button_group
		(GTK_RADIO_BUTTON (notrecv_radiobtn));
	gtk_widget_show (notrecv_radiobtn);
	gtk_box_pack_start (GTK_BOX (vbox2), notrecv_radiobtn, FALSE, FALSE, 0);
	gtk_signal_connect
		(GTK_OBJECT (notrecv_radiobtn), "toggled",
		 GTK_SIGNAL_FUNC (prefs_filter_notrecv_radio_button_toggled),
		 NULL);

	/* register / substitute / delete */

	reg_hbox = gtk_hbox_new (FALSE, 4);
	gtk_widget_show (reg_hbox);
	gtk_box_pack_start (GTK_BOX (vbox1), reg_hbox, FALSE, FALSE, 0);

	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
	gtk_widget_show (arrow);
	gtk_box_pack_start (GTK_BOX (reg_hbox), arrow, FALSE, FALSE, 0);
	gtk_widget_set_usize (arrow, -1, 16);

	btn_hbox = gtk_hbox_new (TRUE, 4);
	gtk_widget_show (btn_hbox);
	gtk_box_pack_start (GTK_BOX (reg_hbox), btn_hbox, FALSE, FALSE, 0);

	reg_btn = gtk_button_new_with_label (_("Register"));
	gtk_widget_show (reg_btn);
	gtk_box_pack_start (GTK_BOX (btn_hbox), reg_btn, FALSE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT (reg_btn), "clicked",
			    GTK_SIGNAL_FUNC (prefs_filter_register_cb), NULL);

	subst_btn = gtk_button_new_with_label (_(" Substitute "));
	gtk_widget_show (subst_btn);
	gtk_box_pack_start (GTK_BOX (btn_hbox), subst_btn, FALSE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT (subst_btn), "clicked",
			    GTK_SIGNAL_FUNC (prefs_filter_substitute_cb),
			    NULL);

	del_btn = gtk_button_new_with_label (_("Delete"));
	gtk_widget_show (del_btn);
	gtk_box_pack_start (GTK_BOX (btn_hbox), del_btn, FALSE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT (del_btn), "clicked",
			    GTK_SIGNAL_FUNC (prefs_filter_delete_cb), NULL);

	cond_hbox = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (cond_hbox);
	gtk_box_pack_start (GTK_BOX (vbox1), cond_hbox, TRUE, TRUE, 0);

	cond_scrolledwin = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (cond_scrolledwin);
	gtk_box_pack_start (GTK_BOX (cond_hbox), cond_scrolledwin,
			    TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (cond_scrolledwin),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);

	cond_clist = gtk_clist_new_with_titles(1, title);
	gtk_widget_show (cond_clist);
	gtk_container_add (GTK_CONTAINER (cond_scrolledwin), cond_clist);
	gtk_clist_set_column_width (GTK_CLIST (cond_clist), 0, 80);
	gtk_clist_set_selection_mode (GTK_CLIST (cond_clist),
				      GTK_SELECTION_BROWSE);
	GTK_WIDGET_UNSET_FLAGS (GTK_CLIST (cond_clist)->column[0].button,
				GTK_CAN_FOCUS);
	gtk_signal_connect (GTK_OBJECT (cond_clist), "select_row",
			    GTK_SIGNAL_FUNC (prefs_filter_select), NULL);

	btn_vbox = gtk_vbox_new (FALSE, 8);
	gtk_widget_show (btn_vbox);
	gtk_box_pack_start (GTK_BOX (cond_hbox), btn_vbox, FALSE, FALSE, 0);
	gtk_widget_set_usize (btn_vbox, 38, -1);

	up_btn = gtk_button_new_with_label (_("Up"));
	gtk_widget_show (up_btn);
	gtk_box_pack_start (GTK_BOX (btn_vbox), up_btn, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (up_btn), "clicked",
			    GTK_SIGNAL_FUNC (prefs_filter_up), NULL);

	down_btn = gtk_button_new_with_label (_("Down"));
	gtk_widget_show (down_btn);
	gtk_box_pack_start (GTK_BOX (btn_vbox), down_btn, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (down_btn), "clicked",
			    GTK_SIGNAL_FUNC (prefs_filter_down), NULL);

	filter.hdr_combo1  = hdr_combo1;
	filter.hdr_combo2  = hdr_combo2;
	filter.hdr_entry1  = GTK_COMBO (hdr_combo1)->entry;
	filter.hdr_entry2  = GTK_COMBO (hdr_combo2)->entry;
	filter.key_entry1  = key_entry1;
	filter.key_entry2  = key_entry2;
	filter.pred_combo1 = pred_combo1;
	filter.pred_combo2 = pred_combo2;
	filter.pred_entry1 = pred_entry1;
	filter.pred_entry2 = pred_entry2;
	filter.op_combo    = op_combo;
	filter.op_entry    = op_entry;

	filter.dest_entry	= dest_entry;
	filter.destsel_btn	= destsel_btn;
	filter.dest_radiobtn	= dest_radiobtn;
	filter.notrecv_radiobtn	= notrecv_radiobtn;
	filter.regex_chkbtn     = regex_chkbtn;

	filter.cond_clist   = cond_clist;
}

static void prefs_display_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *frame_font;
	GtkWidget *table1;
	GtkWidget *label_msgfont;
	GtkWidget *entry_msgfont;
	GtkWidget *button_msgfont;

	GtkWidget *chkbtn_folder_unread;
	GtkWidget *hbox1;
	GtkWidget *chkbtn_enablecol;
	GtkWidget *button_edit_col;

	GtkWidget *chkbtn_transhdr;
	GtkWidget *chkbtn_mbalnum;
	GtkWidget *chkbtn_swapfrom;
	GtkWidget *hbox_linespc;
	GtkWidget *label_linespc;
	GtkObject *spinbtn_linespc_adj;
	GtkWidget *spinbtn_linespc;
	GtkWidget *chkbtn_headspc;
	GtkWidget *chkbtn_disphdr;

	GtkWidget *button_dispitem;

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	PACK_FRAME(vbox1, frame_font, _("Font"));

	table1 = gtk_table_new (1, 3, FALSE);
	gtk_widget_show (table1);
	gtk_container_add (GTK_CONTAINER (frame_font), table1);
	gtk_container_set_border_width (GTK_CONTAINER (table1), 8);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 8);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 8);

	label_msgfont = gtk_label_new (_("Message"));
	gtk_widget_show (label_msgfont);
	gtk_table_attach (GTK_TABLE (table1), label_msgfont, 0, 1, 0, 1,
			  GTK_FILL, (GTK_EXPAND | GTK_FILL), 0, 0);

	entry_msgfont = gtk_entry_new ();
	gtk_widget_show (entry_msgfont);
	gtk_table_attach (GTK_TABLE (table1), entry_msgfont, 1, 2, 0, 1,
			  (GTK_EXPAND | GTK_FILL), 0, 0, 0);

	button_msgfont = gtk_button_new_with_label ("... ");
	gtk_widget_show (button_msgfont);
	gtk_table_attach (GTK_TABLE (table1), button_msgfont, 2, 3, 0, 1,
			  0, 0, 0, 0);
	gtk_signal_connect (GTK_OBJECT (button_msgfont), "clicked",
			    GTK_SIGNAL_FUNC (prefs_font_select), NULL);

	/* ---- Folder View ---- */

	PACK_CHECK_BUTTON (vbox1, chkbtn_folder_unread,
			   _("Display unread number next to folder name"));

	/* ---- Message ---- */

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, TRUE, 0);

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, TRUE, 0);

	PACK_CHECK_BUTTON (hbox1, chkbtn_enablecol,
			   _("Enable coloration of message"));

	button_edit_col = gtk_button_new_with_label (_(" Edit... "));
	gtk_widget_show (button_edit_col);
	gtk_box_pack_end (GTK_BOX (hbox1), button_edit_col, FALSE, TRUE, 0);

	//SET_TOGGLE_SENSITIVITY(chkbtn_enablecol, button_edit_col);
	gtk_widget_set_sensitive(button_edit_col, FALSE);

	PACK_CHECK_BUTTON
		(vbox2, chkbtn_mbalnum,
		 _("Display 2-byte alphabet and numeric with 1-byte character"));
	PACK_CHECK_BUTTON(vbox2, chkbtn_disphdr,
			  _("Display short headers on message view"));

	hbox1 = gtk_hbox_new (FALSE, 32);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, TRUE, 0);

	hbox_linespc = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (hbox1), hbox_linespc, FALSE, TRUE, 0);

	label_linespc = gtk_label_new (_("Line space"));
	gtk_widget_show (label_linespc);
	gtk_box_pack_start (GTK_BOX (hbox_linespc), label_linespc,
			    FALSE, FALSE, 0);

	spinbtn_linespc_adj = gtk_adjustment_new (2, 0, 16, 1, 1, 16);
	spinbtn_linespc = gtk_spin_button_new
		(GTK_ADJUSTMENT (spinbtn_linespc_adj), 1, 0);
	gtk_widget_show (spinbtn_linespc);
	gtk_box_pack_start (GTK_BOX (hbox_linespc), spinbtn_linespc,
			    FALSE, FALSE, 0);
	gtk_widget_set_usize (spinbtn_linespc, 64, -1);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_linespc), TRUE);

	label_linespc = gtk_label_new (_("pixel(s)"));
	gtk_widget_show (label_linespc);
	gtk_box_pack_start (GTK_BOX (hbox_linespc), label_linespc,
			    FALSE, FALSE, 0);

	PACK_CHECK_BUTTON(hbox1, chkbtn_headspc, _("Leave space on head"));

	/* ---- Summary ---- */

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, TRUE, 0);

	PACK_CHECK_BUTTON
		(vbox2, chkbtn_transhdr,
		 _("Translate header name (such as `From:', `Subject:')"));
	PACK_CHECK_BUTTON
		(vbox2, chkbtn_swapfrom,
		 _("Display recipient on `From' column if sender is yourself"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 0);

	button_dispitem = gtk_button_new_with_label
		(_(" Set display item of summary... "));
	gtk_widget_show (button_dispitem);
	gtk_box_pack_start (GTK_BOX (hbox1), button_dispitem, FALSE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT (button_dispitem), "clicked",
			    GTK_SIGNAL_FUNC (prefs_summary_display_item_set),
			    NULL);

	display.entry_msgfont	= entry_msgfont;
	display.button_msgfont	= button_msgfont;

	display.chkbtn_folder_unread = chkbtn_folder_unread;

	display.chkbtn_enablecol = chkbtn_enablecol;
	display.button_edit_col  = button_edit_col;

	display.chkbtn_transhdr = chkbtn_transhdr;
	display.chkbtn_mbalnum  = chkbtn_mbalnum;
	display.chkbtn_swapfrom = chkbtn_swapfrom;
	display.spinbtn_linespc = spinbtn_linespc;
	display.chkbtn_headspc  = chkbtn_headspc;
	display.chkbtn_disphdr  = chkbtn_disphdr;
}

#ifdef USE_GPGME
static void prefs_privacy_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *checkbtn_default_encrypt;
	GtkWidget *checkbtn_default_sign;
	GtkWidget *checkbtn_passphrase_grab;

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);

	PACK_CHECK_BUTTON (vbox2, checkbtn_default_encrypt,
			   _("Default to encrypt in Compose dialog"));

	PACK_CHECK_BUTTON (vbox2, checkbtn_default_sign,
                           _("Default to sign message"));

	PACK_CHECK_BUTTON (vbox2, checkbtn_passphrase_grab,
                           _("Grab input while entering a passphrase"));

	privacy.checkbtn_default_encrypt = checkbtn_default_encrypt;
	privacy.checkbtn_default_sign    = checkbtn_default_sign;
	privacy.checkbtn_passphrase_grab = checkbtn_passphrase_grab;
}
#endif /*USE_GPGME*/


static void prefs_interface_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *checkbtn_emacs;
	GtkWidget *checkbtn_openunread;

	GtkWidget *frame_exit;
	GtkWidget *vbox_exit;
	GtkWidget *hbox1;
	GtkWidget *checkbtn_confonexit;
	GtkWidget *checkbtn_cleanonexit;
	GtkWidget *checkbtn_askonclean;

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);

	PACK_CHECK_BUTTON (vbox2, checkbtn_emacs,
			   _("Emulate the behavior of mouse operation of\n"
			     "Emacs-based mailer"));
	gtk_label_set_justify (GTK_LABEL (GTK_BIN (checkbtn_emacs)->child),
			       GTK_JUSTIFY_LEFT);

	PACK_CHECK_BUTTON
		(vbox2, checkbtn_openunread,
		 _("Open first unread message when entering a folder"));

	PACK_FRAME (vbox1, frame_exit, _("On exit"));

	vbox_exit = gtk_vbox_new (FALSE, VSPACING_NARROW);
	gtk_widget_show (vbox_exit);
	gtk_container_add (GTK_CONTAINER (frame_exit), vbox_exit);
	gtk_container_set_border_width (GTK_CONTAINER (vbox_exit), 8);

	PACK_CHECK_BUTTON (vbox_exit, checkbtn_confonexit,
			   _("Confirm on exit"));

	hbox1 = gtk_hbox_new (FALSE, 32);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox_exit), hbox1, FALSE, FALSE, 0);

	PACK_CHECK_BUTTON (hbox1, checkbtn_cleanonexit,
			   _("Clean trash on exit"));
	PACK_CHECK_BUTTON (hbox1, checkbtn_askonclean,
			   _("Ask before cleaning"));
	SET_TOGGLE_SENSITIVITY (checkbtn_cleanonexit, checkbtn_askonclean);

	interface.checkbtn_emacs	= checkbtn_emacs;
	interface.checkbtn_openunread	= checkbtn_openunread;
	interface.checkbtn_confonexit	= checkbtn_confonexit;
	interface.checkbtn_cleanonexit	= checkbtn_cleanonexit;
	interface.checkbtn_askonclean	= checkbtn_askonclean;
}

static void prefs_other_create(void)
{
	GtkWidget *vbox1;
	GtkWidget *hbox1;

	GtkWidget *uri_frame;
	GtkWidget *uri_label;
	GtkWidget *uri_combo;
	GtkWidget *uri_entry;

	GtkWidget *print_frame;
	GtkWidget *printcmd_label;
	GtkWidget *printcmd_entry;

	GtkWidget *exteditor_frame;
	GtkWidget *exteditor_label;
	GtkWidget *exteditor_combo;
	GtkWidget *exteditor_entry;

	vbox1 = gtk_vbox_new (FALSE, VSPACING);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);

	PACK_FRAME(vbox1, uri_frame,
		   _("External Web browser (%s will be replaced with URI)"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_container_add (GTK_CONTAINER (uri_frame), hbox1);
	gtk_container_set_border_width (GTK_CONTAINER (hbox1), 8);

	uri_label = gtk_label_new (_("Command"));
	gtk_widget_show(uri_label);
	gtk_box_pack_start (GTK_BOX (hbox1), uri_label, FALSE, TRUE, 0);

	uri_combo = gtk_combo_new ();
	gtk_widget_show (uri_combo);
	gtk_box_pack_start (GTK_BOX (hbox1), uri_combo, TRUE, TRUE, 0);
	gtkut_combo_set_items (GTK_COMBO (uri_combo),
			       "netscape -remote 'openURL(%s,raise)'",
			       "gnome-moz-remote --raise --newwin '%s'",
			       "kterm -e w3m '%s'",
			       "kterm -e lynx '%s'",
			       NULL);

	uri_entry = GTK_COMBO (uri_combo)->entry;

	PACK_FRAME(vbox1, print_frame,
		   _("Printing (%s will be replaced with file name)"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_container_add (GTK_CONTAINER (print_frame), hbox1);
	gtk_container_set_border_width (GTK_CONTAINER (hbox1), 8);

	printcmd_label = gtk_label_new (_("Command"));
	gtk_widget_show (printcmd_label);
	gtk_box_pack_start (GTK_BOX (hbox1), printcmd_label, FALSE, FALSE, 0);

	printcmd_entry = gtk_entry_new ();
	gtk_widget_show (printcmd_entry);
	gtk_box_pack_start (GTK_BOX (hbox1), printcmd_entry, TRUE, TRUE, 0);

	PACK_FRAME(vbox1, exteditor_frame,
		   _("External editor (%s will be replaced with file name)"));

	hbox1 = gtk_hbox_new (FALSE, 8);
	gtk_widget_show (hbox1);
	gtk_container_add (GTK_CONTAINER (exteditor_frame), hbox1);
	gtk_container_set_border_width (GTK_CONTAINER (hbox1), 8);

	exteditor_label = gtk_label_new (_("Command"));
	gtk_widget_show (exteditor_label);
	gtk_box_pack_start (GTK_BOX (hbox1), exteditor_label, FALSE, FALSE, 0);

	exteditor_combo = gtk_combo_new ();
	gtk_widget_show (exteditor_combo);
	gtk_box_pack_start (GTK_BOX (hbox1), exteditor_combo, TRUE, TRUE, 0);
	gtkut_combo_set_items (GTK_COMBO (exteditor_combo),
			       "gedit %s",
			       "mgedit --no-fork %s",
			       "emacs %s",
			       "xemacs %s",
			       "kterm -e jed %s",
			       "kterm -e vi %s",
			       NULL);
	exteditor_entry = GTK_COMBO (exteditor_combo)->entry;

	other.uri_combo = uri_combo;
	other.uri_entry = uri_entry;
	other.printcmd_entry = printcmd_entry;

	other.exteditor_combo = exteditor_combo;
	other.exteditor_entry = exteditor_entry;
}

static void prefs_quote_description(void)
{
	if (!quote_desc_win)
		prefs_quote_description_create();

	if (GTK_WIDGET_VISIBLE(quote_desc_win))
		gtk_widget_hide(quote_desc_win);
	else
		gtk_widget_show(quote_desc_win);
}

static void prefs_quote_description_create(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;
	/* GtkWidget *ok_btn; */

	quote_desc_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(quote_desc_win),
			     _("Description of symbols"));
	gtk_container_set_border_width(GTK_CONTAINER(quote_desc_win), 8);
	gtk_signal_connect(GTK_OBJECT(quote_desc_win), "delete_event",
			   GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete), NULL);

	vbox = gtk_vbox_new(FALSE, 8);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(quote_desc_win), vbox);

	hbox = gtk_hbox_new(FALSE, 4);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	label = gtk_label_new
		("%d:\n"
		 "%f:\n"
		 "%n:\n"
		 "%N:\n"
		 "%I:\n"
		 "%s:\n"
		 "%t:\n"
		 "%i:\n"
		 "%%:");
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);

	label = gtk_label_new
		(_("Date\n"
		   "From\n"
		   "Full Name of Sender\n"
		   "First Name of Sender\n"
		   "Initial of Sender\n"
		   "Subject\n"
		   "To\n"
		   "Message-ID\n"
		   "%"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);

#if 0
	ok_btn = gtk_button_new_with_label(_("OK"));
	gtk_widget_show(ok_btn);
	gtk_box_pack_end(GTK_BOX(vbox), ok_btn, FALSE, FALSE, 0);
	gtk_signal_connect_object(GTK_OBJECT(ok_btn), "clicked",
				  GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete),
				  GTK_OBJECT(quote_desc_win));
#endif
}

/* filtering functions */

static void prefs_filter_read_config(void)
{
	gchar *rcpath;
	FILE *fp;
	gchar buf[PREFSBUFSIZE];
	Filter *flt;

	debug_print(_("Reading filter configuration...\n"));

	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, FILTER_RC, NULL);
	if ((fp = fopen(rcpath, "r")) == NULL) {
		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
		g_free(rcpath);
		return;
	}
	g_free(rcpath);

	/* remove all previous filter list */
	while (prefs_common.fltlist != NULL) {
		flt = (Filter *)prefs_common.fltlist->data;
		filter_free(flt);
		prefs_common.fltlist = g_slist_remove(prefs_common.fltlist,
						      flt);
	}

	while (fgets(buf, sizeof(buf), fp) != NULL) {
		g_strchomp(buf);
		flt = filter_read_str(buf);
		if (flt) {
			prefs_common.fltlist =
				g_slist_append(prefs_common.fltlist, flt);
		}
	}

	fclose(fp);
}

static void prefs_filter_write_config(void)
{
	gchar *rcpath;
	PrefFile *pfile;
	GSList *cur;

	debug_print(_("Writing filter configuration...\n"));

	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, FILTER_RC, NULL);
	if ((pfile = prefs_write_open(rcpath)) == NULL) {
		g_warning(_("failed to write configuration to file\n"));
		g_free(rcpath);
		return;
	}

	for (cur = prefs_common.fltlist; cur != NULL; cur = cur->next) {
		Filter *flt = (Filter *)cur->data;
		gchar *fstr;

		fstr = filter_get_str(flt);
		if (fputs(fstr, pfile->fp) == EOF ||
		    fputc('\n', pfile->fp) == EOF) {
			FILE_OP_ERROR(rcpath, "fputs || fputc");
			prefs_write_close_revert(pfile);
			g_free(rcpath);
			g_free(fstr);
			return;
		}
		g_free(fstr);
	}

	g_free(rcpath);

	if (prefs_write_close(pfile) < 0) {
		g_warning(_("failed to write configuration to file\n"));
		return;
	}
}

static void prefs_filter_set_dialog(void)
{
	GtkCList *clist = GTK_CLIST(filter.cond_clist);
	GSList *cur;
	gchar *cond_str[1];
	gint row;

	gtk_clist_freeze(clist);
	gtk_clist_clear(clist);

	cond_str[0] = _("(New)");
	row = gtk_clist_append(clist, cond_str);
	gtk_clist_set_row_data(clist, row, NULL);

	for (cur = prefs_common.fltlist; cur != NULL; cur = cur->next) {
		Filter *flt = (Filter *)cur->data;

		cond_str[0] = filter_get_str(flt);
		subst_char(cond_str[0], '\t', ':');
		row = gtk_clist_append(clist, cond_str);
		gtk_clist_set_row_data(clist, row, flt);

		g_free(cond_str[0]);
	}

	gtk_clist_thaw(clist);
}

static void prefs_filter_set_list(void)
{
	gint row = 1;
	Filter *flt;

	g_slist_free(prefs_common.fltlist);
	prefs_common.fltlist = NULL;

	while ((flt = gtk_clist_get_row_data(GTK_CLIST(filter.cond_clist),
		row)) != NULL) {
		prefs_common.fltlist = g_slist_append(prefs_common.fltlist,
						      flt);
		row++;
	}
}

#define GET_ENTRY(entry) \
	entry_text = gtk_entry_get_text(GTK_ENTRY(entry))

static gint prefs_filter_clist_set_row(gint row)
{
	GtkCList *clist = GTK_CLIST(filter.cond_clist);
	Filter *flt;
	gchar *entry_text;
	gchar *cond_str[1];

	g_return_val_if_fail(row != 0, -1);

	if (GTK_WIDGET_IS_SENSITIVE(filter.dest_entry))
		GET_ENTRY(filter.dest_entry);
	else
		entry_text = FILTER_NOT_RECEIVE;
	if (entry_text[0] == '\0') {
		alertpanel_error(_("Destination is not set."));
		return -1;
	}
	GET_ENTRY(filter.hdr_entry1);
	if (entry_text[0] == '\0') {
		alertpanel_error(_("Header name is not set."));
		return -1;
	}

	flt = g_new0(Filter, 1);

	flt->name1 = g_strdup(entry_text);

	GET_ENTRY(filter.key_entry1);
	if (entry_text[0] != '\0')
		flt->body1 = g_strdup(entry_text);

	GET_ENTRY(filter.hdr_entry2);
	if (entry_text[0] != '\0' && strcmp(entry_text, _("(none)")) != 0) {
		flt->name2 = g_strdup(entry_text);

		GET_ENTRY(filter.key_entry2);
		if (entry_text[0] != '\0')
			flt->body2 = g_strdup(entry_text);
	}

	GET_ENTRY(filter.pred_entry1);
	if (!strcmp(entry_text, _("contains")))
		flt->flag1 = FLT_CONTAIN;
	GET_ENTRY(filter.pred_entry2);
	if (!strcmp(entry_text, _("contains")))
		flt->flag2 = FLT_CONTAIN;

	GET_ENTRY(filter.op_entry);
	if (!strcmp(entry_text, "and"))
		flt->cond = FLT_AND;
	else
		flt->cond = FLT_OR;

	if (GTK_WIDGET_IS_SENSITIVE(filter.dest_entry)) {
		entry_text = gtk_entry_get_text(GTK_ENTRY(filter.dest_entry));
		flt->dest = g_strdup(entry_text);
		flt->action = FLT_MOVE;
	} else
		flt->action = FLT_NOTRECV;

	cond_str[0] = filter_get_str(flt);
	subst_char(cond_str[0], '\t', ':');

	if (row < 0)
		row = gtk_clist_append(clist, cond_str);
	else {
		Filter *tmpflt;

		gtk_clist_set_text(clist, row, 0, cond_str[0]);
		tmpflt = gtk_clist_get_row_data(clist, row);
		if (tmpflt)
			filter_free(tmpflt);
	}

	gtk_clist_set_row_data(clist, row, flt);

	g_free(cond_str[0]);

	prefs_filter_set_list();

	return row;
}

static void prefs_filter_select_dest_cb(void)
{
	FolderItem *dest;

	dest = foldersel_folder_sel(NULL);
	if (!dest) return;

	gtk_entry_set_text(GTK_ENTRY(filter.dest_entry), dest->path);
}

static void prefs_filter_register_cb(void)
{
	prefs_filter_clist_set_row(-1);
}

static void prefs_filter_substitute_cb(void)
{
	GtkCList *clist = GTK_CLIST(filter.cond_clist);
	Filter *flt;
	gint row;

	if (!clist->selection) return;

	row = GPOINTER_TO_INT(clist->selection->data);
	if (row == 0) return;

	flt = gtk_clist_get_row_data(clist, row);
	if (!flt) return;

	prefs_filter_clist_set_row(row);
}

static void prefs_filter_delete_cb(void)
{
	GtkCList *clist = GTK_CLIST(filter.cond_clist);
	Filter *flt;
	gint row;

	if (!clist->selection) return;
	row = GPOINTER_TO_INT(clist->selection->data);
	if (row == 0) return;

	if (alertpanel(_("Delete rule"),
		       _("Do you really want to delete this rule?"),
		       _("Yes"), _("No"), NULL) == G_ALERTALTERNATE)
		return;

	flt = gtk_clist_get_row_data(clist, row);
	filter_free(flt);
	gtk_clist_remove(clist, row);
	prefs_common.fltlist = g_slist_remove(prefs_common.fltlist, flt);
}

static void prefs_filter_up(void)
{
	GtkCList *clist = GTK_CLIST(filter.cond_clist);
	gint row;

	if (!clist->selection) return;

	row = GPOINTER_TO_INT(clist->selection->data);
	if (row > 1) {
		gtk_clist_row_move(clist, row, row - 1);
		prefs_filter_set_list();
	}
}

static void prefs_filter_down(void)
{
	GtkCList *clist = GTK_CLIST(filter.cond_clist);
	gint row;

	if (!clist->selection) return;

	row = GPOINTER_TO_INT(clist->selection->data);
	if (row > 0 && row < clist->rows - 1) {
		gtk_clist_row_move(clist, row, row + 1);
		prefs_filter_set_list();
	}
}

#define ENTRY_SET_TEXT(entry, str) \
	gtk_entry_set_text(GTK_ENTRY(entry), str ? str : "")

static void prefs_filter_select(GtkCList *clist, gint row, gint column,
				GdkEvent *event)
{
	Filter *flt;
	Filter default_flt = {"Subject", NULL, _("(none)"), NULL,
			      FLT_CONTAIN, FLT_CONTAIN, FLT_AND,
			      NULL, FLT_MOVE};

	flt = gtk_clist_get_row_data(clist, row);
	if (!flt)
		flt = &default_flt;

	ENTRY_SET_TEXT(filter.dest_entry, flt->dest);
	ENTRY_SET_TEXT(filter.hdr_entry1, flt->name1);
	ENTRY_SET_TEXT(filter.key_entry1, flt->body1);
	ENTRY_SET_TEXT(filter.hdr_entry2,
		       flt->name2 ? flt->name2 : _("(none)"));
	ENTRY_SET_TEXT(filter.key_entry2, flt->body2);

	ENTRY_SET_TEXT(filter.pred_entry1,
		       FLT_IS_CONTAIN(flt->flag1)
		       ? _("contains") : _("not contain"));
	ENTRY_SET_TEXT(filter.pred_entry2,
		       FLT_IS_CONTAIN(flt->flag2)
		       ? _("contains") : _("not contain"));

	gtk_entry_set_text(GTK_ENTRY(filter.op_entry),
			   flt->cond == FLT_OR ? "or" : "and");
	if (flt->action == FLT_NOTRECV)
		gtk_toggle_button_set_active
			(GTK_TOGGLE_BUTTON(filter.notrecv_radiobtn), TRUE);
	else
		gtk_toggle_button_set_active
			(GTK_TOGGLE_BUTTON(filter.dest_radiobtn), TRUE);
}

static void prefs_filter_dest_radio_button_toggled(void)
{
	gtk_widget_set_sensitive(filter.dest_entry, TRUE);
	gtk_widget_set_sensitive(filter.destsel_btn, TRUE);
}

static void prefs_filter_notrecv_radio_button_toggled(void)
{
	gtk_widget_set_sensitive(filter.dest_entry, FALSE);
	gtk_widget_set_sensitive(filter.destsel_btn, FALSE);
}

/* functions for setting items of SummaryView */

static struct _SummaryDisplayItem
{
	GtkWidget *window;

	GtkWidget *chkbtn[N_SUMMARY_COLS];

	GtkWidget *ok_btn;
	GtkWidget *cancel_btn;
} summarydispitem;

#define SET_ACTIVE(column, var) \
	gtk_toggle_button_set_active \
		(GTK_TOGGLE_BUTTON(summarydispitem.chkbtn[column]), \
		 prefs_common.var)
#define GET_ACTIVE(column, var) \
	prefs_common.var = gtk_toggle_button_get_active \
		(GTK_TOGGLE_BUTTON(summarydispitem.chkbtn[column]))

void prefs_summary_display_item_set(void)
{
	static gboolean cancelled;

	if (!summarydispitem.window)
		prefs_summary_display_item_dialog_create(&cancelled);
	gtk_widget_grab_focus(summarydispitem.ok_btn);
	gtk_widget_show(summarydispitem.window);
	manage_window_set_transient(GTK_WINDOW(summarydispitem.window));

	SET_ACTIVE(S_COL_MARK, show_mark);
	SET_ACTIVE(S_COL_UNREAD, show_unread);
	SET_ACTIVE(S_COL_MIME, show_mime);
	SET_ACTIVE(S_COL_NUMBER, show_number);
	SET_ACTIVE(S_COL_SIZE, show_size);
	SET_ACTIVE(S_COL_DATE, show_date);
	SET_ACTIVE(S_COL_FROM, show_from);
	SET_ACTIVE(S_COL_SUBJECT, show_subject);

	gtk_main();
	gtk_widget_hide(summarydispitem.window);

	if (cancelled != TRUE) {
		GET_ACTIVE(S_COL_MARK, show_mark);
		GET_ACTIVE(S_COL_UNREAD, show_unread);
		GET_ACTIVE(S_COL_MIME, show_mime);
		GET_ACTIVE(S_COL_NUMBER, show_number);
		GET_ACTIVE(S_COL_SIZE, show_size);
		GET_ACTIVE(S_COL_DATE, show_date);
		GET_ACTIVE(S_COL_FROM, show_from);
		GET_ACTIVE(S_COL_SUBJECT, show_subject);

		main_window_reflect_prefs_all();
	}
}

#define SET_CHECK_BUTTON(column, label) \
{ \
	summarydispitem.chkbtn[column] = \
		gtk_check_button_new_with_label(label); \
	gtk_box_pack_start(GTK_BOX(chkbtn_vbox), \
			   summarydispitem.chkbtn[column], \
			   FALSE, FALSE, 0); \
}

static void prefs_summary_display_item_dialog_create(gboolean *cancelled)
{
	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *chkbtn_vbox;
	GtkWidget *hbbox;
	GtkWidget *ok_btn;
	GtkWidget *cancel_btn;

	window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
	gtk_window_set_title(GTK_WINDOW(window), _("Set display item"));
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			   GTK_SIGNAL_FUNC(display_item_delete_event),
			   cancelled);
	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
			   GTK_SIGNAL_FUNC(display_item_key_pressed),
			   cancelled);

	vbox = gtk_vbox_new(FALSE, 8);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	chkbtn_vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), chkbtn_vbox, FALSE, FALSE, 0);

	SET_CHECK_BUTTON(S_COL_MARK, _("Mark"));
	SET_CHECK_BUTTON(S_COL_UNREAD, _("Unread"));
	SET_CHECK_BUTTON(S_COL_MIME, _("MIME"));
	SET_CHECK_BUTTON(S_COL_NUMBER, _("Number"));
	SET_CHECK_BUTTON(S_COL_SIZE, _("Size"));
	SET_CHECK_BUTTON(S_COL_DATE, _("Date"));
	SET_CHECK_BUTTON(S_COL_FROM, _("From"));
	SET_CHECK_BUTTON(S_COL_SUBJECT, _("Subject"));

	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
				&cancel_btn, _("Cancel"), NULL, NULL);
	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
	gtk_widget_grab_default(ok_btn);

	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
			   GTK_SIGNAL_FUNC(display_item_ok), cancelled);
	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
			   GTK_SIGNAL_FUNC(display_item_cancel), cancelled);

	gtk_widget_show_all(vbox);

	summarydispitem.window = window;
	summarydispitem.ok_btn = ok_btn;
	summarydispitem.cancel_btn = cancel_btn;
}

static void display_item_ok(GtkWidget *widget, gboolean *cancelled)
{
	*cancelled = FALSE;
	gtk_main_quit();
}

static void display_item_cancel(GtkWidget *widget, gboolean *cancelled)
{
	*cancelled = TRUE;
	gtk_main_quit();
}

static gint display_item_delete_event(GtkWidget *widget, GdkEventAny *event,
				      gboolean *cancelled)
{
	*cancelled = TRUE;
	gtk_main_quit();

	return TRUE;
}

static void display_item_key_pressed(GtkWidget *widget, GdkEventKey *event,
				     gboolean *cancelled)
{
	if (event && event->keyval == GDK_Escape) {
		*cancelled = TRUE;
		gtk_main_quit();
	}
}

static void prefs_font_select(GtkButton *button)
{
	if (!font_sel_win) {
		font_sel_win = gtk_font_selection_dialog_new
			(_("Font selection"));
		gtk_window_position(GTK_WINDOW(font_sel_win),
				    GTK_WIN_POS_CENTER);
		gtk_signal_connect(GTK_OBJECT(font_sel_win), "delete_event",
				   GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete),
				   NULL);
		gtk_signal_connect
			(GTK_OBJECT(font_sel_win), "key_press_event",
			 GTK_SIGNAL_FUNC(prefs_font_selection_key_pressed),
			 NULL);
		gtk_signal_connect
			(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(font_sel_win)->ok_button),
			 "clicked",
			 GTK_SIGNAL_FUNC(prefs_font_selection_ok),
			 NULL);
		gtk_signal_connect_object
			(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(font_sel_win)->cancel_button),
			 "clicked",
			 GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete),
			 GTK_OBJECT(font_sel_win));
	}

	manage_window_set_transient(GTK_WINDOW(font_sel_win));
	gtk_window_set_modal(GTK_WINDOW(font_sel_win), TRUE);
	gtk_widget_grab_focus
		(GTK_FONT_SELECTION_DIALOG(font_sel_win)->ok_button);
	gtk_widget_show(font_sel_win);
}

static void prefs_font_selection_key_pressed(GtkWidget *widget,
					     GdkEventKey *event,
					     gpointer data)
{
	if (event && event->keyval == GDK_Escape)
		gtk_widget_hide(font_sel_win);
}

static void prefs_font_selection_ok(GtkButton *button)
{
	gchar *fontname;

	fontname = gtk_font_selection_dialog_get_font_name
		(GTK_FONT_SELECTION_DIALOG(font_sel_win));

	if (fontname) {
		gtk_entry_set_text(GTK_ENTRY(display.entry_msgfont), fontname);
		g_free(fontname);
	}

	gtk_widget_hide(font_sel_win);
}

static void prefs_common_charset_set_data_from_optmenu(PrefParam *pparam)
{
	GtkWidget *menu;
	GtkWidget *menuitem;
	gchar *charset;

	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
	menuitem = gtk_menu_get_active(GTK_MENU(menu));
	charset = gtk_object_get_user_data(GTK_OBJECT(menuitem));
	g_free(*((gchar **)pparam->data));
	*((gchar **)pparam->data) = g_strdup(charset);
}

static void prefs_common_charset_set_optmenu(PrefParam *pparam)
{
	GList *cur;
	GtkOptionMenu *optmenu = GTK_OPTION_MENU(*pparam->widget);
	GtkWidget *menu;
	GtkWidget *menuitem;
	gchar *charset;
	gint n = 0;

	g_return_if_fail(optmenu != NULL);
	g_return_if_fail(*((gchar **)pparam->data) != NULL);

	menu = gtk_option_menu_get_menu(optmenu);
	for (cur = GTK_MENU_SHELL(menu)->children;
	     cur != NULL; cur = cur->next) {
		menuitem = GTK_WIDGET(cur->data);
		charset = gtk_object_get_user_data(GTK_OBJECT(menuitem));
		if (!strcmp(charset, *((gchar **)pparam->data))) {
			gtk_option_menu_set_history(optmenu, n);
			return;
		}
		n++;
	}

	gtk_option_menu_set_history(optmenu, 0);
	prefs_common_charset_set_data_from_optmenu(pparam);
}

static void prefs_common_key_pressed(GtkWidget *widget, GdkEventKey *event,
				     gpointer data)
{
	if (event && event->keyval == GDK_Escape)
		gtk_widget_hide(dialog.window);
}

static void prefs_common_ok(GtkButton *button)
{
	prefs_common_apply(button);
	gtk_widget_hide(dialog.window);
	if (quote_desc_win && GTK_WIDGET_VISIBLE(quote_desc_win))
		gtk_widget_hide(quote_desc_win);
}

static void prefs_common_apply(GtkButton *button)
{
	prefs_set_data_from_dialog(param);
	//messageview_set_font();
	prefs_common_save_config();
	prefs_filter_write_config();
}
