/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999,2000 Hiroyuki Yamamoto
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __SOCKET_H__
#define __SOCKET_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>

#if USE_THREADS
#  include <pthread.h>
#endif

#ifndef SOCKINFO_DEFINED
#define SOCKINFO_DEFINED
typedef struct _SockInfo SockInfo;
#endif

typedef enum
{
	CONN_READY,
	CONN_LOOKUPSUCCESS,
	CONN_ESTABLISHED,
	CONN_LOOKUPFAILED,
	CONN_FAILED
} ConnectionState;

struct _SockInfo
{
        GIOChannel *channel;
	gchar *hostname;
	gushort port;
	ConnectionState state;
	gpointer data;
        char *buf;
        int buflen;
#if USE_THREADS
	pthread_t connect_thr;
	pthread_mutex_t mutex;
#endif
};

SockInfo *sock_connect_nb	(const gchar *hostname, gushort port);
SockInfo *sock_connect		(const gchar *hostname, gushort port);

#if USE_THREADS
SockInfo *sock_connect_with_thread  (const gchar *hostname, gushort port);
#endif

gint sock_write		(SockInfo *sockinfo, const gchar *buf, gint len);
gint sock_read		(SockInfo *sockinfo, gchar *buf, gint len);
gboolean sock_bytes_pending (SockInfo *sockinfo);
gboolean sock_lines_pending (SockInfo *sockinfo);
gint sock_close		(SockInfo *sockinfo);

gint sock_printf	(SockInfo *sockinfo, const gchar *format, ...)
			 G_GNUC_PRINTF(2, 3);
gint sock_puts		(SockInfo *sockinfo, const gchar *buf);

#endif /* __SOCKET_H__ */



